@@ -214,7 +214,13 @@ class CourseAdmin(FrontendEditableAdminMixin, PageExtensionAdmin):
     """Admin class for the Course model"""
 
     list_display = ["title", "is_listed"]
-    frontend_editable_fields = ("code", "duration", "effort", "is_listed")
+    frontend_editable_fields = (
+        "code",
+        "duration",
+        "effort",
+        "is_listed",
+        "is_self_paced",
+    )
 
     # pylint: disable=no-self-use
     def title(self, obj):
