@@ -1,22 +1,23 @@
 """
 Declare and configure the models for the courses application
 """
+# pylint: disable=too-many-lines
 from collections.abc import Mapping
 from datetime import MAXYEAR, datetime
 
 from django.conf import settings
 from django.core.exceptions import ValidationError
 from django.db import models
-from django.db.models import Prefetch, Q
+from django.db.models import Q
 from django.urls import reverse
-from django.utils import timezone, translation
-from django.utils.functional import lazy
+from django.utils import timezone
+from django.utils.functional import cached_property, lazy
 from django.utils.translation import gettext_lazy as _
 
 import pytz
 from cms.constants import PUBLISHER_STATE_DIRTY
 from cms.extensions.extension_pool import extension_pool
-from cms.models import Page, PagePermission, Title
+from cms.models import Page, PagePermission
 from cms.models.pluginmodel import CMSPlugin
 from filer.fields.image import FilerImageField
 from filer.models import FolderPermission
@@ -24,15 +25,13 @@ from parler.models import TranslatableModel, TranslatedField, TranslatedFieldsMo
 
 from ...core.defaults import ALL_LANGUAGES
 from ...core.fields.duration import CompositeDurationField
-from ...core.fields.effort import EffortField
 from ...core.fields.multiselect import MultiSelectField
 from ...core.helpers import get_permissions
-from ...core.models import BasePageExtension, PagePluginMixin
-from .. import defaults
-from .category import Category
-from .organization import Organization
-from .person import Person
-from .program import Program
+from ...core.models import BasePageExtension
+from .. import defaults, utils
+from .category import Category, CategoryPluginModel
+from .organization import Organization, OrganizationPluginModel
+from .person import Person, PersonPluginModel
 from .role import PageRole
 
 MAX_DATE = datetime(MAXYEAR, 12, 31, tzinfo=pytz.utc)
@@ -44,30 +43,33 @@ class CourseState(Mapping):
     (
         ONGOING_OPEN,
         FUTURE_OPEN,
+        ARCHIVED_OPEN,
         FUTURE_NOT_YET_OPEN,
         FUTURE_CLOSED,
         ONGOING_CLOSED,
-        ARCHIVED,
+        ARCHIVED_CLOSED,
         TO_BE_SCHEDULED,
-    ) = range(7)
+    ) = range(8)
 
     STATE_CALLS_TO_ACTION = {
         ONGOING_OPEN: _("enroll now"),
         FUTURE_OPEN: _("enroll now"),
+        ARCHIVED_OPEN: _("study now"),
         FUTURE_NOT_YET_OPEN: None,
         FUTURE_CLOSED: None,
         ONGOING_CLOSED: None,
-        ARCHIVED: None,
+        ARCHIVED_CLOSED: None,
         TO_BE_SCHEDULED: None,
     }
 
     STATE_TEXTS = {
         ONGOING_OPEN: _("closing on"),
         FUTURE_OPEN: _("starting on"),
+        ARCHIVED_OPEN: _("closing on"),
         FUTURE_NOT_YET_OPEN: _("starting on"),
         FUTURE_CLOSED: _("enrollment closed"),
         ONGOING_CLOSED: _("on-going"),
-        ARCHIVED: _("archived"),
+        ARCHIVED_CLOSED: _("archived"),
         TO_BE_SCHEDULED: _("to be scheduled"),
     }
 
@@ -78,27 +80,34 @@ class CourseState(Mapping):
         Several states are possible for a course run each of which is given a priority. The
         lower the priority, the more interesting the course run is (a course run open for
         enrollment is more interesting than an archived course run):
-          0: a run is on-going and open for enrollment > "closing on": {enrollment_end}
-          1: a run is future and open for enrollment > "starting on": {start}
-          2: a run is future and not yet open for enrollment > "starting on": {start}
-          3: a run is future and no more open for enrollment > "closed": {None}
-          4: a run is on-going but closed for enrollment > "on going": {None}
-          5: there's a finished run in the past > "archived": {None}
-          6: theres's no run with a start date or no run at all > "to be scheduled": {None}
+        - ONGOING_OPEN: a run is on-going and open for enrollment > "closing on": {enrollment_end}
+        - FUTURE_OPEN: a run is future and open for enrollment > "starting on": {start}
+        - ARCHIVED_OPEN: a run is past but open for enrollment > "closing on": {enrollment_end}
+        - FUTURE_NOT_YET_OPEN: a run is future and not yet open for enrollment
+            > "starting on": {start}
+        - FUTURE_CLOSED: a run is future and no more open for enrollment > "closed": {None}
+        - ONGOING_CLOSED: a run is on-going but closed for enrollment > "on going": {None}
+        - ARCHIVED_CLOSED: there's a finished run in the past > "archived": {None}
+        - TO_BE_SCHEDULED: theres's no run with a start date or no run at all
+            > "to be scheduled": {None}
         """
         # Check that `date_time` is set when it should be
         if date_time is None and priority in [
             CourseState.ONGOING_OPEN,
             CourseState.FUTURE_OPEN,
+            CourseState.ARCHIVED_OPEN,
             CourseState.FUTURE_NOT_YET_OPEN,
         ]:
             raise ValidationError(
-                "date_time should not be null for a {:d} course state.".format(priority)
+                f"date_time should not be null for a {priority:d} course state."
             )
 
         # A special case of being open is when enrollment never ends
         text = self.STATE_TEXTS[priority]
-        if priority == CourseState.ONGOING_OPEN and date_time.year == MAXYEAR:
+        if (
+            priority in [CourseState.ONGOING_OPEN, CourseState.ARCHIVED_OPEN]
+            and date_time.year == MAXYEAR
+        ):
             text = _("forever open")
             date_time = None
         kwargs = {
@@ -126,6 +135,7 @@ class CourseState(Mapping):
         return self._d["priority"] < other["priority"]
 
 
+# pylint: disable=too-many-public-methods
 class Course(BasePageExtension):
     """
     The course page extension represents and records a course in the catalog.
@@ -144,27 +154,37 @@ class Course(BasePageExtension):
         null=True,
         blank=True,
     )
-    effort = EffortField(
-        time_units=defaults.TIME_UNITS,
-        default_effort_unit=defaults.DEFAULT_EFFORT_UNIT,
-        default_reference_unit=defaults.DEFAULT_REFERENCE_UNIT,
-        max_length=80,
-        blank=True,
-        null=True,
-    )
+
     duration = CompositeDurationField(
         time_units=defaults.TIME_UNITS,
         default_unit=defaults.DEFAULT_TIME_UNIT,
         max_length=80,
         blank=True,
         null=True,
+        help_text=_("The course time range."),
     )
+
+    effort = CompositeDurationField(
+        time_units=defaults.EFFORT_UNITS,
+        default_unit=defaults.DEFAULT_EFFORT_UNIT,
+        max_length=80,
+        blank=True,
+        null=True,
+        help_text=_("Total amount of time to complete this course."),
+    )
+
     is_listed = models.BooleanField(
         default=True,
         verbose_name=_("is listed"),
         help_text=_("Tick if you want the course to be visible on the search page."),
     )
 
+    is_self_paced = models.BooleanField(
+        default=False,
+        verbose_name=_("is self paced"),
+        help_text=_("Tick if the course pace is self paced."),
+    )
+
     PAGE = defaults.COURSES_PAGE
 
     class Meta:
@@ -174,10 +194,24 @@ class Course(BasePageExtension):
 
     def __str__(self):
         """Human representation of a course."""
-        return "{model}: {title}".format(
-            model=self._meta.verbose_name.title(),
-            title=self.extended_object.get_title(),
-        )
+        model = self._meta.verbose_name.title()
+        title = self.extended_object.get_title()
+        return f"{model:s}: {title:s}"
+
+    def clean(self):
+        """
+        We normalize the code with slugify for better uniqueness
+        """
+        # Normalize the code by slugifying and capitalizing it
+        self.code = utils.normalize_code(self.code)
+        return super().clean()
+
+    @property
+    def pt_effort(self):
+        """Return effort as a PT string for schema.org metadata."""
+        (effort, effort_unit) = self.effort
+        unit_letter = effort_unit[0].upper()
+        return f"PT{effort:d}{unit_letter:s}"
 
     def validate_unique(self, exclude=None):
         """
@@ -188,20 +222,26 @@ class Course(BasePageExtension):
         which is not possible.
         """
         super().validate_unique()
-        if (
-            self.extended_object.publisher_is_draft
-            and not self.is_snapshot
-            and self.__class__.objects.exclude(pk=self.pk)
-            .filter(
+
+        if self.code and not self.is_snapshot:
+            # Check unicity for the version being saved (draft or published)
+            is_draft = self.extended_object.publisher_is_draft
+            uniqueness_query = self.__class__.objects.filter(
                 code=self.code,
-                # allow public duplicates
-                extended_object__publisher_is_draft=True,
+                extended_object__publisher_is_draft=is_draft,
                 # allow snapshot duplicates
                 extended_object__node__parent__cms_pages__course__isnull=True,
             )
-            .exists()
-        ):
-            raise ValidationError({"code": ["Code must be unique."]})
+
+            # If the page is being updated, we should exclude it while looking for duplicates
+            if self.pk:
+                uniqueness_query = uniqueness_query.exclude(pk=self.pk)
+
+            # Raise a ValidationError if the code already exists
+            if uniqueness_query.exists():
+                raise ValidationError(
+                    {"code": ["A course already exists with this code."]}
+                )
 
     def get_admin_url_to_add_run(self, request):
         """
@@ -298,55 +338,14 @@ class Course(BasePageExtension):
             ),
         )
 
-    def get_persons(self, language=None):
-        """
-        Return the persons linked to the course via a person plugin in any of the
-        placeholders on the course detail page, ranked by their `path` to respect
-        the order in the persons tree.
-        """
-        language = language or translation.get_language()
-
-        selector = "extended_object__person_plugins__cmsplugin_ptr"
-        filter_dict = {
-            "{:s}__language".format(selector): language,
-            "{:s}__placeholder__page".format(selector): self.extended_object,
-        }
-        # For a public course, we must filter out persons that are not published in
-        # any language
-        if self.extended_object.publisher_is_draft is False:
-            filter_dict["extended_object__title_set__published"] = True
-
-        return (
-            Person.objects.filter(**filter_dict)
-            .select_related("extended_object")
-            .order_by("extended_object__node__path")
-            .distinct()
-        )
-
     def get_organizations(self, language=None):
         """
         Return the organizations linked to the course via an organization plugin in any
         of the placeholders on the course detail page, ranked by their `path` to respect
         the order in the organizations tree.
         """
-        language = language or translation.get_language()
-
-        selector = "extended_object__organization_plugins__cmsplugin_ptr"
-        # pylint: disable=no-member
-        filter_dict = {
-            "{:s}__language".format(selector): language,
-            "{:s}__placeholder__page".format(selector): self.extended_object,
-        }
-        # For a public course, we must filter out organizations that are not published in
-        # any language
-        if self.extended_object.publisher_is_draft is False:
-            filter_dict["extended_object__title_set__published"] = True
-
-        return (
-            Organization.objects.filter(**filter_dict)
-            .select_related("extended_object")
-            .order_by("extended_object__node__path")
-            .distinct()
+        return self.get_direct_related_page_extensions(
+            Organization, OrganizationPluginModel, language=language
         )
 
     def get_main_organization(self):
@@ -365,33 +364,27 @@ class Course(BasePageExtension):
             .first()
         )
 
+    def get_persons(self, language=None):
+        """
+        Return the persons linked to the course via a person plugin in any of the
+        placeholders on the course detail page, ranked by their `path` to respect
+        the order in the persons tree.
+        """
+        return self.get_direct_related_page_extensions(
+            Person, PersonPluginModel, language=language
+        )
+
     def get_categories(self, language=None):
         """
         Return the categories linked to the course via a category plugin in any of the
         placeholders on the course detail page, ranked by their `path` to respect the
         order in the categories tree.
         """
-        language = language or translation.get_language()
-
-        selector = "extended_object__category_plugins__cmsplugin_ptr"
-        # pylint: disable=no-member
-        filter_dict = {
-            "{:s}__language".format(selector): language,
-            "{:s}__placeholder__page".format(selector): self.extended_object,
-        }
-        # For a public course, we must filter out categories that are not published in
-        # any language
-        if self.extended_object.publisher_is_draft is False:
-            filter_dict["extended_object__title_set__published"] = True
-
-        return (
-            Category.objects.filter(**filter_dict)
-            .select_related("extended_object")
-            .order_by("extended_object__node__path")
-            .distinct()
+        return self.get_direct_related_page_extensions(
+            Category, CategoryPluginModel, language=language
         )
 
-    def get_root_to_leaf_category_pages(self):
+    def get_root_to_leaf_public_category_pages(self):
         """
         Build a query retrieving all pages linked to this course's categories or one of their
         ancestors excluding the meta category itself.
@@ -403,7 +396,9 @@ class Course(BasePageExtension):
         category_query = self.get_categories()
 
         # 1. We want the pages directly related to a category of the course
-        page_query = Page.objects.filter(publisher_draft__category__in=category_query)
+        page_query = Page.objects.filter(
+            publisher_draft__category__in=category_query, title_set__published=True
+        )
         # 2. We want the pages related to one of the ancestors of the categories of the course
         for category in category_query.select_related(
             "public_extension__extended_object"
@@ -414,7 +409,8 @@ class Course(BasePageExtension):
             # Don't include the meta category as it materializes a "filter bank" and not a
             # search option
             page_query = page_query | ancestor_query.filter(
-                node__parent__cms_pages__category__isnull=False
+                node__parent__cms_pages__category__isnull=False,
+                title_set__published=True,
             )
         return page_query.distinct()
 
@@ -447,7 +443,8 @@ class Course(BasePageExtension):
 
         return query
 
-    def get_course_runs(self):
+    @cached_property
+    def course_runs(self):
         """
         Returns a query yielding the course runs related to the course. They may be directly
         related to the course or to a snapshot of the course.
@@ -464,48 +461,108 @@ class Course(BasePageExtension):
             direct_course__extended_object__publisher_is_draft=is_draft,
         ).order_by("-start")
 
-    def get_course_runs_dict(self):
+    @property
+    def course_runs_dict(self):
         """Returns a dict of course runs grouped by their state."""
         course_runs_dict = {
             i: [] for i in range(len(CourseState.STATE_CALLS_TO_ACTION))
         }
-        for run in self.get_course_runs():
+        for run in self.course_runs:
             course_runs_dict[run.state["priority"]].append(run)
 
         return dict(course_runs_dict)
 
+    def get_pace(self):
+        """
+        Returns the course pace computed according to duration and effort fields or
+        None if one of these fields is blank or if `is_self_paced` flag is equal to False.
+
+        This method returns a tuple of three values:
+        - pace : the float value of the pace (ratio between effort / duration)
+        - pace_unit
+        - pace_reference_unit
+        """
+        if self.is_self_paced is True or self.duration is None:
+            return None
+
+        if self.effort is None:
+            raise ValueError("Cannot compute pace without effort.")
+
+        (effort, effort_unit) = self.effort
+        (duration, duration_unit) = self.duration
+
+        time_units_keys = list(dict(defaults.TIME_UNITS))
+        if time_units_keys.index(effort_unit) >= time_units_keys.index(duration_unit):
+            raise ValueError(
+                (
+                    "Cannot compute pace with effort unit less than or equal to the duration unit."
+                    f" ({effort_unit}/{duration_unit})"
+                )
+            )
+
+        pace = effort / duration
+        return (pace, effort_unit, duration_unit)
+
+    def get_pace_display(self):
+        """
+        Returns a human readable pace value to display.
+        If the course is self paced or has no duration, it returns `Self paced`.
+        If pace cannot be computed, it returns None.
+
+        To improve readability, we manage three cases with different renders:
+        - Pace is a full hour -> ~2 hours/day
+        - Pace is more than a full hour -> ~2h30/day
+        - Pace is under an hour -> ~45 minutes/day
+
+        Furthermore, as this information is an estimation, we do not want the
+        value be too precise, so we round minutes to fifteen minutes slots.
+        """
+        try:
+            (pace, pace_unit, pace_reference_unit) = self.get_pace()
+        except TypeError:
+            return _("Self paced")
+        except ValueError:
+            return None
+
+        time_units = defaults.TIME_UNITS
+        pace_in_minutes = pace * 60 if pace_unit == defaults.HOUR else pace
+        pace_hours = int(pace_in_minutes // 60)
+        pace_minutes = int(pace_in_minutes % 60)
+        pace_minutes_rounded_by_quarter = (
+            round(pace_minutes / 15) * 15
+        )  # round minutes by fifteen
+
+        # Pace is under an hour
+        if pace_hours == 0:
+            resolution = 15 if pace_minutes > 15 else 5
+            return _("~{pace:d} {effort_unit!s}/{duration_unit!s}").format(
+                pace=round(pace_minutes / resolution) * resolution
+                or 5,  # Display at least 5 minutes
+                effort_unit=time_units[defaults.MINUTE][1],
+                duration_unit=time_units[pace_reference_unit][0],
+            )
+        # Pace is a full hour
+        if pace_minutes_rounded_by_quarter == 0:
+            count_index = 1 if pace_hours > 1 else 0
+            return _("~{pace:d} {effort_unit!s}/{duration_unit!s}").format(
+                pace=pace_hours,
+                effort_unit=time_units[defaults.HOUR][count_index],
+                duration_unit=time_units[pace_reference_unit][0],
+            )
+
+        # Pace is more than a full hour
+        return _("~{pace_hours:d}h{pace_minutes:d}/{duration_unit!s}").format(
+            pace_hours=pace_hours,
+            pace_minutes=pace_minutes_rounded_by_quarter,
+            duration_unit=time_units[pace_reference_unit][0],
+        )
+
     def get_programs(self, language=None):
         """
         Return a query to get the programs related to this course ie for which a plugin for
         this course is linked to the program page via any placeholder.
         """
-        is_draft = self.extended_object.publisher_is_draft
-        course = self if is_draft else self.draft_extension
-        language = language or translation.get_language()
-
-        bfs = (
-            "extended_object__placeholders__cmsplugin__courses_coursepluginmodel__page"
-        )
-        filter_dict = {
-            "extended_object__publisher_is_draft": is_draft,
-            "extended_object__placeholders__cmsplugin__language": language,
-            bfs: course.extended_object,
-        }
-
-        # pylint: disable=no-member
-        return (
-            Program.objects.filter(**filter_dict)
-            .select_related("extended_object")
-            .prefetch_related(
-                Prefetch(
-                    "extended_object__title_set",
-                    to_attr="prefetched_titles",
-                    queryset=Title.objects.filter(language=language),
-                )
-            )
-            .order_by("extended_object__node__path")
-            .distinct()
-        )
+        return self.get_reverse_related_page_extensions("program", language=language)
 
     @property
     def state(self):
@@ -517,7 +574,7 @@ class Course(BasePageExtension):
         # The default state is for a course that has no course runs
         best_state = CourseState(CourseState.TO_BE_SCHEDULED)
 
-        for course_run in self.get_course_runs().only(
+        for course_run in self.course_runs.only(
             "start", "end", "enrollment_start", "enrollment_end"
         ):
             state = course_run.state
@@ -634,7 +691,7 @@ class CourseRun(TranslatableModel):
 
     def __str__(self):
         """Human representation of a course run."""
-        start = "{:%y/%m/%d %H:%M} - ".format(self.start) if self.start else ""
+        start = f"{self.start:%y/%m/%d %H:%M} - " if self.start else ""
         return f"Course run {self.id!s} starting {start:s}"
 
     def copy_translations(self, oldinstance, language=None):
@@ -735,31 +792,34 @@ class CourseRun(TranslatableModel):
         A static method not using the instance allows to call it with an Elasticsearch result.
         """
         if not start or not enrollment_start:
-            return CourseState(6)
+            return CourseState(CourseState.TO_BE_SCHEDULED)
 
         # course run end dates are not required and should default to forever
         # e.g. a course run with no end date is presumed to be always on-going
         end = end or MAX_DATE
-        enrollment_end = enrollment_end or end
+        enrollment_end = enrollment_end or MAX_DATE
 
         now = timezone.now()
         if start < now:
             if end > now:
                 if enrollment_end > now:
                     # ongoing open
-                    return CourseState(0, enrollment_end)
+                    return CourseState(CourseState.ONGOING_OPEN, enrollment_end)
                 # ongoing closed
-                return CourseState(4)
-            # archived
-            return CourseState(5)
+                return CourseState(CourseState.ONGOING_CLOSED)
+            if enrollment_start < now < enrollment_end:
+                # archived open
+                return CourseState(CourseState.ARCHIVED_OPEN, enrollment_end)
+            # archived closed
+            return CourseState(CourseState.ARCHIVED_CLOSED)
         if enrollment_start > now:
             # future not yet open
-            return CourseState(2, start)
+            return CourseState(CourseState.FUTURE_NOT_YET_OPEN, start)
         if enrollment_end > now:
             # future open
-            return CourseState(1, start)
+            return CourseState(CourseState.FUTURE_OPEN, start)
         # future already closed
-        return CourseState(3)
+        return CourseState(CourseState.FUTURE_CLOSED)
 
     @property
     def state(self):
@@ -807,9 +867,8 @@ class CourseRunTranslation(TranslatedFieldsModel):
 
     def __str__(self):
         """Human representation of a course run translation."""
-        return "{model}: {name}".format(
-            model=self._meta.verbose_name.title(), name=self.title
-        )
+        model = self._meta.verbose_name.title()
+        return f"{model:s}: {self.title:s}"
 
     # pylint: disable=signature-differs
     def save(self, *args, **kwargs):
@@ -834,7 +893,7 @@ class CourseRunTranslation(TranslatedFieldsModel):
         return super().save(*args, **kwargs)
 
 
-class CoursePluginModel(PagePluginMixin, CMSPlugin):
+class CoursePluginModel(CMSPlugin):
     """
     Course plugin model handles the relation from CoursePlugin
     to their Course instance
@@ -871,6 +930,10 @@ class CoursePluginModel(PagePluginMixin, CMSPlugin):
         verbose_name = _("course plugin")
         verbose_name_plural = _("course plugins")
 
+    def __str__(self):
+        """Human representation of a page plugin"""
+        return self.page.get_title()
+
 
 class Licence(TranslatableModel):
     """
@@ -899,9 +962,8 @@ class Licence(TranslatableModel):
 
     def __str__(self):
         """Human representation of a licence."""
-        return "{model}: {name}".format(
-            model=self._meta.verbose_name.title(), name=self.name
-        )
+        model = self._meta.verbose_name.title()
+        return f"{model:s}: {self.name:s}"
 
 
 class LicenceTranslation(TranslatedFieldsModel):
@@ -923,9 +985,8 @@ class LicenceTranslation(TranslatedFieldsModel):
 
     def __str__(self):
         """Human representation of a licence translation."""
-        return "{model}: {name}".format(
-            model=self._meta.verbose_name.title(), name=self.name
-        )
+        model = self._meta.verbose_name.title()
+        return f"{model:s}: {self.name:s}"
 
 
 class LicencePluginModel(CMSPlugin):
@@ -943,9 +1004,8 @@ class LicencePluginModel(CMSPlugin):
 
     def __str__(self):
         """Human representation of a licence plugin."""
-        return "{model:s}: {name:s}".format(
-            model=self._meta.verbose_name.title(), name=self.licence.name
-        )
+        model = self._meta.verbose_name.title()
+        return f"{model:s}: {self.licence.name:s}"
 
 
 extension_pool.register(Course)
