@@ -1,23 +1,20 @@
 """
 Declare and configure the models for the courses application
 """
-from django.apps import apps
 from django.conf import settings
 from django.core.exceptions import ValidationError
 from django.db import models
-from django.db.models import Prefetch
-from django.utils import translation
-from django.utils.text import slugify
+from django.db.models import Q
 from django.utils.translation import gettext_lazy as _
 
 from cms.api import Page, PagePermission
 from cms.extensions.extension_pool import extension_pool
-from cms.models import CMSPlugin, Title
+from cms.models import CMSPlugin
 from filer.models import FolderPermission
 
 from ...core.helpers import get_permissions
-from ...core.models import BasePageExtension, PagePluginMixin
-from .. import defaults
+from ...core.models import BasePageExtension
+from .. import defaults, utils
 from .role import PageRole
 
 
@@ -31,8 +28,10 @@ class Organization(BasePageExtension):
     page to present the organization.
     """
 
+    CODE_MAX_LENGTH = 100
+
     code = models.CharField(
-        _("code"), db_index=True, max_length=100, null=True, blank=True
+        _("code"), db_index=True, max_length=CODE_MAX_LENGTH, null=True, blank=True
     )
 
     PAGE = defaults.ORGANIZATIONS_PAGE
@@ -45,11 +44,9 @@ class Organization(BasePageExtension):
 
     def __str__(self):
         """Human representation of an organization"""
-        return "{model}: {name} ({code})".format(
-            code=self.code,
-            name=self.extended_object.get_title(),
-            model=self._meta.verbose_name.title(),
-        )
+        model = self._meta.verbose_name.title()
+        name = self.extended_object.get_title()
+        return f"{model:s}: {name:s} ({self.code:s})"
 
     def get_es_id(self):
         """
@@ -66,9 +63,7 @@ class Organization(BasePageExtension):
         """
         We normalize the code with slugify for better uniqueness
         """
-        if self.code:
-            # Normalize the code by slugifying and capitalizing it
-            self.code = slugify(self.code, allow_unicode=True).upper()
+        self.code = utils.normalize_code(self.code)
         return super().clean()
 
     def validate_unique(self, exclude=None):
@@ -90,7 +85,7 @@ class Organization(BasePageExtension):
             # Raise a ValidationError if the code already exists
             if uniqueness_query.exists():
                 raise ValidationError(
-                    {"code": ["An Organization already exists with this code."]}
+                    {"code": ["An organization already exists with this code."]}
                 )
         return super().validate_unique(exclude=exclude)
 
@@ -159,67 +154,43 @@ class Organization(BasePageExtension):
         Return a query to get the courses related to this organization ie for which a plugin for
         this organization is linked to the course page via any placeholder.
         """
-        is_draft = self.extended_object.publisher_is_draft
-        organization = self if is_draft else self.draft_extension
-        language = language or translation.get_language()
-
-        bfs = "extended_object__placeholders__cmsplugin__courses_organizationpluginmodel__page"
-        filter_dict = {
-            "extended_object__node__parent__cms_pages__course__isnull": True,
-            "extended_object__publisher_is_draft": is_draft,
-            "extended_object__placeholders__cmsplugin__language": language,
-            bfs: organization.extended_object,
-        }
-
-        course_model = apps.get_model(app_label="courses", model_name="course")
-        # pylint: disable=no-member
-        return (
-            course_model.objects.filter(**filter_dict)
-            .select_related("extended_object")
-            .prefetch_related(
-                Prefetch(
-                    "extended_object__title_set",
-                    to_attr="prefetched_titles",
-                    queryset=Title.objects.filter(language=language),
-                )
-            )
-            .order_by("extended_object__node__path")
-            .distinct()
-        )
+        return self.get_reverse_related_page_extensions(
+            "course", language=language
+        ).filter(extended_object__node__parent__cms_pages__course__isnull=True)
 
     def get_persons(self, language=None):
         """
         Return a query to get the persons related to this organization ie for which a plugin for
         this organization is linked to the person page via any placeholder.
         """
-        is_draft = self.extended_object.publisher_is_draft
-        organization = self if is_draft else self.draft_extension
-        language = language or translation.get_language()
+        return self.get_reverse_related_page_extensions("person", language=language)
 
-        bfs = "extended_object__placeholders__cmsplugin__courses_organizationpluginmodel__page"
-        filter_dict = {
-            "extended_object__publisher_is_draft": is_draft,
-            "extended_object__placeholders__cmsplugin__language": language,
-            bfs: organization.extended_object,
-        }
-
-        person_model = apps.get_model(app_label="courses", model_name="person")
-        # pylint: disable=no-member
+    @staticmethod
+    def get_organizations_codes(page, language: str):
+        """
+        Return organization attached to the page and
+        organizations linked to the current page via an organization plugin in any of the
+        placeholders on the page.
+        """
         return (
-            person_model.objects.filter(**filter_dict)
-            .select_related("extended_object")
-            .prefetch_related(
-                Prefetch(
-                    "extended_object__title_set",
-                    to_attr="prefetched_titles",
-                    queryset=Title.objects.filter(language=language),
+            Organization.objects.filter(
+                Q(
+                    extended_object__organization_plugins__cmsplugin_ptr__language=language,
+                    extended_object__organization_plugins__cmsplugin_ptr__placeholder__page=page,  # noqa pylint: disable=line-too-long
                 )
+                | Q(
+                    extended_object__exact=page,
+                ),
+                extended_object__title_set__published=True,
+                code__isnull=False,
             )
             .distinct()
+            .order_by("code")
+            .values_list("code", flat=True)
         )
 
 
-class OrganizationPluginModel(PagePluginMixin, CMSPlugin):
+class OrganizationPluginModel(CMSPlugin):
     """
     Organization plugin model handles the relation from OrganizationPlugin
     to their Organization instance
@@ -248,8 +219,12 @@ class OrganizationPluginModel(PagePluginMixin, CMSPlugin):
         verbose_name = _("organization plugin")
         verbose_name_plural = _("organization plugins")
 
+    def __str__(self):
+        """Human representation of a page plugin"""
+        return self.page.get_title()
 
-class OrganizationsByCategoryPluginModel(PagePluginMixin, CMSPlugin):
+
+class OrganizationsByCategoryPluginModel(CMSPlugin):
     """
     Handle the relation between a OrganizationsByCategoryPlugin plugin and its Category
     instance.
