@@ -3,7 +3,7 @@ Import custom settings and set up defaults for values the Search app needs
 """
 from django.conf import settings
 from django.utils.functional import lazy
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 # Elasticsearch
 ES_CHUNK_SIZE = 500
@@ -12,6 +12,11 @@ ES_PAGE_SIZE = 10
 # Use a lazy to enable easier testing by not defining the value at bootstrap time
 ES_INDICES_PREFIX = lazy(lambda: settings.RICHIE_ES_INDICES_PREFIX)()
 
+# Define which analyzer should be used for each language
+QUERY_ANALYZERS = getattr(
+    settings, "RICHIE_QUERY_ANALYZERS", {"en": "english", "fr": "french"}
+)
+
 # Define the scoring boost (in ElasticSearch) related value names receive when using
 # full-text search.
 # For example, when a user searches for "Science" in full-text, it should match any
@@ -20,11 +25,21 @@ ES_INDICES_PREFIX = lazy(lambda: settings.RICHIE_ES_INDICES_PREFIX)()
 # This lower score factor is the boost value we get or set here.
 RELATED_CONTENT_BOOST = 0.05
 
-FACET_SORTING_DEFAULT = "conf"
+FACET_SORTING_DEFAULT = "count"
 
 FACET_COUNTS_DEFAULT_LIMIT = getattr(settings, "RICHIE_FACET_COUNTS_DEFAULT_LIMIT", 10)
 FACET_COUNTS_MAX_LIMIT = getattr(settings, "RICHIE_FACET_COUNTS_MAX_LIMIT", 50)
 
+ES_STATE_WEIGHTS = getattr(settings, "RICHIE_ES_STATE_WEIGHTS", None) or [
+    80,  # ONGOING_OPEN
+    70,  # FUTURE_OPEN
+    60,  # ARCHIVED_OPEN
+    30,  # FUTURE_NOT_YET_OPEN
+    6,  # FUTURE_CLOSED
+    5,  # ONGOING_CLOSED
+    1,  # ARCHIVED_CLOSED
+]
+
 FILTERS_CONFIGURATION = [
     (
         "richie.apps.search.filter_definitions.StaticChoicesFilterDefinition",
@@ -34,6 +49,7 @@ FILTERS_CONFIGURATION = [
             "min_doc_count": 0,
             "name": "new",
             "position": 0,
+            "sorting": "conf",
             "values": {"new": _("First session")},
         },
     ),
@@ -50,6 +66,7 @@ FILTERS_CONFIGURATION = [
                         "min_doc_count": 0,
                         "name": "availability",
                         "position": 1,
+                        "sorting": "conf",
                     },
                 ),
                 (
@@ -61,7 +78,6 @@ FILTERS_CONFIGURATION = [
                         "min_doc_count": 1,
                         "name": "languages",
                         "position": 5,
-                        "sorting": "count",
                     },
                 ),
             ],
