@@ -26,7 +26,6 @@ class OrganizationsIndexer:
     objects into what we want to index in ElasticSearch
     """
 
-    document_type = "organization"
     index_name = f"{ES_INDICES_PREFIX}_organizations"
     form = ItemSearchForm
     mapping = {
@@ -35,7 +34,7 @@ class OrganizationsIndexer:
             # Searchable
             # description & title are handled by `MULTILINGUAL_TEXT`
             **{
-                "complete.{:s}".format(lang): {
+                f"complete.{lang:s}": {
                     "type": "completion",
                     "analyzer": "simple_diacritics_insensitive",
                 }
@@ -44,24 +43,16 @@ class OrganizationsIndexer:
             # Not searchable
             "absolute_url": {"type": "object", "enabled": False},
             "logo": {"type": "object", "enabled": False},
+            # Create a raw title field to enable alphabetical sorting
+            # We cannot use the default title field as the analysis prevents sorting on it
+            **{
+                f"title_raw.{lang}": {"type": "keyword"}
+                for lang, _ in settings.LANGUAGES
+            },
         },
     }
     scripts = {}
-    display_fields = ["absolute_url", "logo", "title.*"]
-
-    @staticmethod
-    def get_es_id(page):
-        """
-        An ID built with the node path and the position of this organization in the taxonomy:
-            - P: parent, the organization has children
-            - L: leaf, the organization has no children
-
-        For example: a parent organization `P_00010002` and its child `L_000100020001`.
-        """
-        return "{type:s}-{path:s}".format(
-            type="L" if page.node.is_leaf() else "P",  # Leaf or Parent?
-            path=page.node.path,
-        )
+    display_fields = ["absolute_url", "logo", "title"]
 
     @classmethod
     def get_es_document_for_organization(cls, organization, index=None, action="index"):
@@ -95,21 +86,23 @@ class OrganizationsIndexer:
             description[simple_text.cmsplugin_ptr.language].append(simple_text.body)
 
         return {
-            "_id": cls.get_es_id(organization.extended_object),
+            "_id": organization.get_es_id(),
             "_index": index,
             "_op_type": action,
-            "_type": cls.document_type,
             "absolute_url": {
-                language: organization.extended_object.get_absolute_url(language)
-                for language in titles.keys()
+                lang: organization.extended_object.get_absolute_url(lang)
+                for lang, _ in settings.LANGUAGES
             },
             "complete": {
                 language: slice_string_for_completion(title)
                 for language, title in titles.items()
             },
             "logo": logo_images,
-            "description": {l: " ".join(st) for l, st in description.items()},
+            "description": {
+                language: " ".join(st) for language, st in description.items()
+            },
             "title": titles,
+            "title_raw": titles,
         }
 
     @classmethod
@@ -119,10 +112,14 @@ class OrganizationsIndexer:
         """
         index = index or cls.index_name
 
-        for organization in Organization.objects.filter(
-            extended_object__publisher_is_draft=False,
-            extended_object__title_set__published=True,
-        ).distinct():
+        for organization in (
+            Organization.objects.filter(
+                extended_object__publisher_is_draft=False,
+                extended_object__title_set__published=True,
+            )
+            .distinct()
+            .iterator()
+        ):
             yield cls.get_es_document_for_organization(
                 organization, index=index, action=action
             )
